"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getVariants = exports.getUpdateVariantVisibilityScript = exports.getUpdateCookieAndStylesScript = exports.getInitVariantsFnsScriptString = exports.checkShouldRenderVariants = void 0;
var _target = require("../../constants/target");
var _isBrowser = require("../../functions/is-browser");
var _inlinedFns = require("./inlined-fns");
/**
 * We hardcode explicit function names here, because the `.toString()` of a function can change depending on the bundler.
 * Some bundlers will minify the fn name, etc.
 *
 * So we hardcode the function names here, and then use those names in the script string to make sure the function names are consistent.
 */
const UPDATE_COOKIES_AND_STYLES_SCRIPT_NAME = 'builderIoAbTest';
const UPDATE_VARIANT_VISIBILITY_SCRIPT_FN_NAME = 'builderIoRenderContent';
const getVariants = content => Object.values((content === null || content === void 0 ? void 0 : content.variations) || {}).map(variant => ({
  ...variant,
  testVariationId: variant.id,
  id: content === null || content === void 0 ? void 0 : content.id
}));
exports.getVariants = getVariants;
const checkShouldRenderVariants = ({
  canTrack,
  content
}) => {
  const hasVariants = getVariants(content).length > 0;

  /**
   * We cannot SSR in React-Native.
   */
  if (_target.TARGET === 'reactNative') return false;
  if (!hasVariants) return false;
  if (!canTrack) return false;

  /**
   * For Vue and Svelte, we need to (initially) render the variants. This is to avoid hydration mismatch errors.
   *
   * Unlike React, Vue's hydration checks are shallow and do not check the attributes/contents of each element, so we
   * are able to modify the `hidden` HTML attributes and `display` CSS properties without causing a hydration mismatch error.
   *
   * NOTE: For Solid/Svelte: after the app is hydrated, we strip the variants from the DOM (on mount) to reduce the amount of HTML in the DOM.
   * This isn't necessary, just a nice to have.
   */
  if (_target.TARGET === 'vue' || _target.TARGET === 'svelte') return true;
  if ((0, _isBrowser.isBrowser)()) return false;
  return true;
};
exports.checkShouldRenderVariants = checkShouldRenderVariants;
const getIsHydrationTarget = target => target === 'react' || target === 'reactNative';
const isAngularSDK = _target.TARGET === 'angular';
const isHydrationTarget = getIsHydrationTarget(_target.TARGET);
const getInitVariantsFnsScriptString = () => `
  window.${UPDATE_COOKIES_AND_STYLES_SCRIPT_NAME} = ${_inlinedFns.UPDATE_COOKIES_AND_STYLES_SCRIPT}
  window.${UPDATE_VARIANT_VISIBILITY_SCRIPT_FN_NAME} = ${_inlinedFns.UPDATE_VARIANT_VISIBILITY_SCRIPT}
  `;
exports.getInitVariantsFnsScriptString = getInitVariantsFnsScriptString;
const getUpdateCookieAndStylesScript = (variants, contentId) => `
  window.${UPDATE_COOKIES_AND_STYLES_SCRIPT_NAME}(
    "${contentId}",${JSON.stringify(variants)}, ${isHydrationTarget}, ${isAngularSDK}
  )`;
exports.getUpdateCookieAndStylesScript = getUpdateCookieAndStylesScript;
const getUpdateVariantVisibilityScript = ({
  contentId,
  variationId
}) => `window.${UPDATE_VARIANT_VISIBILITY_SCRIPT_FN_NAME}(
    "${variationId}", "${contentId}", ${isHydrationTarget}
  )`;
exports.getUpdateVariantVisibilityScript = getUpdateVariantVisibilityScript;
//# sourceMappingURL=helpers.js.map