"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.SDKS_SUPPORTING_PERSONALIZATION = exports.SDKS_REQUIRING_RESET_APPROACH = exports.DEFAULT_INDEX = void 0;
exports.checkShouldRenderVariants = checkShouldRenderVariants;
Object.defineProperty(exports, "filterWithCustomTargeting", {
  enumerable: true,
  get: function () {
    return _inlinedFns.filterWithCustomTargeting;
  }
});
exports.getBlocksToRender = getBlocksToRender;
exports.getUpdateVisibilityStylesScript = exports.getPersonalizationScript = exports.getInitPersonalizationVariantsFnsScriptString = void 0;
var _target = require("../../constants/target");
var _isBrowser = require("../../functions/is-browser");
var _isEditing = require("../../functions/is-editing");
var _inlinedFns = require("./helpers/inlined-fns");
const DEFAULT_INDEX = exports.DEFAULT_INDEX = 'default';
const FILTER_WITH_CUSTOM_TARGETING_SCRIPT_FN_NAME = 'filterWithCustomTargeting';
const BUILDER_IO_PERSONALIZATION_SCRIPT_FN_NAME = 'builderIoPersonalization';
const UPDATE_VARIANT_VISIBILITY_SCRIPT_FN_NAME = 'updateVisibilityStylesScript';
/**
 * SDKs that support Variant Containers
 */
const SDKS_SUPPORTING_PERSONALIZATION = exports.SDKS_SUPPORTING_PERSONALIZATION = ['react', 'vue', 'svelte', 'qwik'];

/**
 * After hydration, we reset the tree
 * These SDKs handle Personalization Container in a special way:
 * - first, the inlined script will help us add `display: none`, `aria-hidden: true` and `hidden: true` to the non-winning variants
 * - then, on mount / when the component is hydrated - we reset the tree with only the winning variant and deleting the entire tree
 *
 * This helps us to avoid flicker and show the correct / winning variant initially and then once we've hydrated we show the winning variant
 * and keep a track of the cookies to update to the correct variant dynamically when the cookie updates.
 */
const SDKS_REQUIRING_RESET_APPROACH = exports.SDKS_REQUIRING_RESET_APPROACH = ['vue', 'svelte', 'qwik'];
function checkShouldRenderVariants(variants, canTrack) {
  const hasVariants = variants && variants.length > 0;
  if (_target.TARGET === 'reactNative') return false;
  if (!hasVariants) return false;
  if (!canTrack) return false;
  if (SDKS_REQUIRING_RESET_APPROACH.includes(_target.TARGET)) return true;
  if ((0, _isBrowser.isBrowser)()) return false;
  return true;
}
function getBlocksToRender({
  variants,
  previewingIndex,
  isHydrated,
  filteredVariants,
  fallbackBlocks
}) {
  const fallback = {
    blocks: fallbackBlocks ?? [],
    path: 'this.children',
    index: DEFAULT_INDEX
  };
  if (isHydrated && (0, _isEditing.isEditing)()) {
    // If editing a specific variant
    if (typeof previewingIndex === 'number' && previewingIndex < ((variants === null || variants === void 0 ? void 0 : variants.length) ?? 0)) {
      const variant = variants === null || variants === void 0 ? void 0 : variants[previewingIndex];
      if (variant) {
        return {
          blocks: variant.blocks,
          path: `variants.${previewingIndex}.blocks`,
          index: previewingIndex
        };
      }
    }
    // Otherwise we're editing the default variant
    return fallback;
  }

  // If we're on the browser, check if there's a winning variant
  if ((0, _isBrowser.isBrowser)()) {
    const winningVariant = filteredVariants === null || filteredVariants === void 0 ? void 0 : filteredVariants[0];
    if (winningVariant && variants) {
      const variantIndex = variants.indexOf(winningVariant);
      if (variantIndex !== -1) {
        return {
          blocks: winningVariant.blocks,
          path: `variants.${variantIndex}.blocks`,
          index: variantIndex
        };
      }
    }
  }

  // If no winning variant or we are on the server, return the default variant
  return fallback;
}
const getInitPersonalizationVariantsFnsScriptString = () => {
  return `
  window.${FILTER_WITH_CUSTOM_TARGETING_SCRIPT_FN_NAME} = ${_inlinedFns.FILTER_WITH_CUSTOM_TARGETING_SCRIPT}
  window.${BUILDER_IO_PERSONALIZATION_SCRIPT_FN_NAME} = ${_inlinedFns.PERSONALIZATION_SCRIPT}
  window.${UPDATE_VARIANT_VISIBILITY_SCRIPT_FN_NAME} = ${_inlinedFns.UPDATE_VISIBILITY_STYLES_SCRIPT}
  `;
};
exports.getInitPersonalizationVariantsFnsScriptString = getInitPersonalizationVariantsFnsScriptString;
const isHydrationTarget = _target.TARGET === 'react';
const getPersonalizationScript = (variants, blockId, locale) => {
  return `window.${BUILDER_IO_PERSONALIZATION_SCRIPT_FN_NAME}(${JSON.stringify(variants)}, "${blockId}", ${isHydrationTarget}${locale ? `, "${locale}"` : ''})`;
};
exports.getPersonalizationScript = getPersonalizationScript;
const getUpdateVisibilityStylesScript = (variants, blockId, locale) => {
  return `window.${UPDATE_VARIANT_VISIBILITY_SCRIPT_FN_NAME}(${JSON.stringify(variants)}, "${blockId}", ${isHydrationTarget}${locale ? `, "${locale}"` : ''})`;
};
exports.getUpdateVisibilityStylesScript = getUpdateVisibilityStylesScript;
//# sourceMappingURL=helpers.js.map