"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.generateContentUrl = void 0;
var _flatten = require("../../helpers/flatten");
var _search = require("../../helpers/search/search");
var _apiVersion = require("../../types/api-version");
var _index = require("../get-builder-search-params/index");
var _isBrowser = require("../is-browser");
const isPositiveNumber = thing => typeof thing === 'number' && !isNaN(thing) && thing >= 0;
const generateContentUrl = options => {
  const {
    limit = 30,
    userAttributes,
    query,
    model,
    apiKey,
    enrich,
    locale,
    apiVersion = _apiVersion.DEFAULT_API_VERSION,
    fields,
    omit,
    offset,
    cacheSeconds,
    staleCacheSeconds,
    sort,
    includeUnpublished,
    apiHost
  } = options;
  if (!apiKey) {
    throw new Error('Missing API key');
  }
  if (!['v3'].includes(apiVersion)) {
    throw new Error(`Invalid apiVersion: expected 'v3', received '${apiVersion}'`);
  }

  // if we are fetching an array of content, we disable noTraverse for perf reasons.
  const noTraverse = limit !== 1;
  const baseUrl = apiHost || 'https://cdn.builder.io';
  const url = new URL(`${baseUrl}/api/${apiVersion}/content/${model}`);
  url.searchParams.set('apiKey', apiKey);
  url.searchParams.set('limit', String(limit));
  url.searchParams.set('noTraverse', String(noTraverse));
  url.searchParams.set('includeRefs', String(true));
  const finalLocale = locale || (userAttributes === null || userAttributes === void 0 ? void 0 : userAttributes.locale);
  let finalUserAttributes = userAttributes || {};
  if (finalLocale) {
    url.searchParams.set('locale', finalLocale);
    finalUserAttributes = {
      locale: finalLocale,
      ...finalUserAttributes
    };
  }
  if (enrich) url.searchParams.set('enrich', String(enrich));
  url.searchParams.set('omit', omit ?? 'meta.componentsUsed');
  if (fields) {
    url.searchParams.set('fields', fields);
  }
  if (Number.isFinite(offset) && offset > -1) {
    url.searchParams.set('offset', String(Math.floor(offset)));
  }
  if (typeof includeUnpublished === 'boolean') {
    url.searchParams.set('includeUnpublished', String(includeUnpublished));
  }
  if (cacheSeconds && isPositiveNumber(cacheSeconds)) {
    url.searchParams.set('cacheSeconds', String(cacheSeconds));
  }
  if (staleCacheSeconds && isPositiveNumber(staleCacheSeconds)) {
    url.searchParams.set('staleCacheSeconds', String(staleCacheSeconds));
  }
  if (sort) {
    const flattened = (0, _flatten.flatten)({
      sort
    });
    for (const key in flattened) {
      url.searchParams.set(key, JSON.stringify(flattened[key]));
    }
  }

  // TODO: how to express 'offset' in the url - as direct queryparam or as flattened in options[key] ?

  const queryOptions = {
    ...(0, _index.getBuilderSearchParamsFromWindow)(),
    ...(0, _search.normalizeSearchParams)(options.options || {})
  };
  finalUserAttributes = {
    ...finalUserAttributes,
    ...getUserAttributesAsJSON(queryOptions)
  };
  const flattened = (0, _flatten.flatten)(queryOptions);
  for (const key in flattened) {
    url.searchParams.set(key, String(flattened[key]));
  }
  if (Object.keys(finalUserAttributes).length > 0) {
    url.searchParams.set('userAttributes', JSON.stringify(finalUserAttributes));
  }
  if (query) {
    const flattened = (0, _flatten.flattenMongoQuery)({
      query
    });
    for (const key in flattened) {
      url.searchParams.set(key, JSON.stringify(flattened[key]));
    }
  }
  return url;
};
exports.generateContentUrl = generateContentUrl;
const getUserAttributesFromQueryOptions = queryOptions => {
  const newUserAttributes = {};
  for (const key in queryOptions) {
    if (key.startsWith('userAttributes.')) {
      newUserAttributes[key] = queryOptions[key];
      delete queryOptions[key];
    }
  }
  return newUserAttributes;
};
const getUserAttributesAsJSON = queryOptions => {
  if ((0, _isBrowser.isBrowser)() && queryOptions['preview'] === 'BUILDER_STUDIO') {
    queryOptions['userAttributes.urlPath'] = window.location.pathname;
    queryOptions['userAttributes.host'] = window.location.host;
    const queryOptionsForUserAttributes = getUserAttributesFromQueryOptions(queryOptions);
    const {
      userAttributes
    } = (0, _flatten.unflatten)(queryOptionsForUserAttributes);
    return userAttributes;
  }
  return {};
};
//# sourceMappingURL=generate-content-url.js.map