"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports._processContentResult = void 0;
exports.fetchEntries = fetchEntries;
exports.fetchOneEntry = fetchOneEntry;
var _target = require("../../constants/target");
var _abTests = require("../../helpers/ab-tests");
var _canTrack = require("../../helpers/canTrack");
var _logger = require("../../helpers/logger");
var _get = require("../../helpers/preview-lru-cache/get");
var _sdkHeaders = require("../../helpers/sdk-headers");
var _getFetch = require("../get-fetch");
var _isBrowser = require("../is-browser");
var _generateContentUrl = require("./generate-content-url");
const checkContentHasResults = content => 'results' in content;

/**
 * Returns the first content entry that matches the given options.
 */
async function fetchOneEntry(options) {
  const allContent = await fetchEntries({
    ...options,
    limit: 1
  });
  if (allContent) {
    return allContent[0] || null;
  }
  return null;
}
const _fetchContent = async options => {
  var _options$fetchOptions;
  const url = (0, _generateContentUrl.generateContentUrl)(options);
  const _fetch = options.fetch ?? _getFetch.fetch;
  const fetchOptions = {
    ...options.fetchOptions,
    headers: {
      ...((_options$fetchOptions = options.fetchOptions) === null || _options$fetchOptions === void 0 ? void 0 : _options$fetchOptions.headers),
      ...(0, _sdkHeaders.getSdkHeaders)()
    }
  };
  const res = await _fetch(url.href, fetchOptions);
  const content = await res.json();
  return content;
};

/**
 * @internal Exported only for testing purposes. Do not use.
 */
const _processContentResult = async (options, content, url = (0, _generateContentUrl.generateContentUrl)(options)) => {
  const canTrack = (0, _canTrack.getDefaultCanTrack)(options.canTrack);
  const isPreviewing = url.search.includes(`preview=`);
  if (_target.TARGET === 'rsc' && isPreviewing) {
    const newResults = [];
    for (const item of content.results) {
      const previewContent = (0, _get.getPreviewContent)(url.searchParams);
      newResults.push(previewContent || item);
    }
    content.results = newResults;
  }
  if (!canTrack) return content.results;
  if (!((0, _isBrowser.isBrowser)() || _target.TARGET === 'reactNative')) return content.results;

  /**
   * For client-side navigations, it is ideal to handle AB testing at this point instead of using our
   * complex multi-rendering variants approach, which is only needed for SSR'd content.
   *
   * This is also where react-native would handle AB testing.
   */
  try {
    const newResults = [];
    for (const item of content.results) {
      newResults.push(await (0, _abTests.handleABTesting)({
        item,
        canTrack
      }));
    }
    content.results = newResults;
  } catch (e) {
    _logger.logger.error('Could not process A/B tests. ', e);
  }
  return content.results;
};

/**
 * Returns a paginated array of entries that match the given options.
 */
exports._processContentResult = _processContentResult;
async function fetchEntries(options) {
  const url = (0, _generateContentUrl.generateContentUrl)(options);
  const content = await _fetchContent(options);
  if (!checkContentHasResults(content)) {
    _logger.logger.error('Error fetching data. ', {
      url,
      content,
      options
    });
    throw content;
  }
  return _processContentResult(options, content);
}
//# sourceMappingURL=index.js.map